from typing import Iterator, Optional, Set, Union

from attr import dataclass
from google.cloud.storage import Bucket, Client
from returns.functions import raise_exception
from returns.result import safe
from typing_extensions import final


@final
@dataclass(frozen=True, slots=True)
class ListBuckets(object):
    """List storage buckets.

    .. code:: python

      >>> from returns.pipeline import is_successful
      >>> assert ListBuckets()()
      >>> assert is_successful(ListBuckets()(prefix='test'))
    """

    _client = Client

    @safe
    def __call__(
        self,
        max_results: Optional[int] = None,
        page_token: Optional[str] = None,
        prefix: Optional[str] = None,
        fields: Optional[Set] = None,
        projection: str = 'noAcl',
        project: Optional[str] = None,
        timeout: Union[float, int] = 60,
    ) -> Set[Bucket]:
        """List the storage bucket files."""
        client = self._initialize_client().alt(  # type: ignore
            raise_exception,
        ).unwrap()

        return client.list_buckets(
            max_results=max_results,
            page_token=page_token,
            prefix=prefix,
            fields=fields,
            projection=projection,
            project=project,
            timeout=timeout,
        )

    @safe
    def _initialize_client(self) -> Client:
        return self._client()


def list_buckets(
    max_results: Optional[int] = None,
    page_token: Optional[str] = None,
    prefix: Optional[str] = None,
    fields: Optional[Set] = None,
    projection: str = 'noAcl',
    project: Optional[str] = None,
    timeout: Union[float, int] = 60,
) -> Bucket:
    """List Buckets, but return Set[Bucket] instead of ResultE[Set[Bucket]].

    Raise exception when Modal is in failure state.
    """
    return ListBuckets()(
        max_results=max_results,
        page_token=page_token,
        prefix=prefix,
        fields=fields,
        projection=projection,
        project=project,
        timeout=timeout,
    ).alt(
        raise_exception,
    ).unwrap()


def list_bucket_names(buckets: Set[Bucket]) -> Iterator[str]:
    """Iterate over Buckets and retrieve their names.

    Raise NoneType Exception when Bucket is None.
    """
    return map(lambda bucket: bucket.name, buckets)



from attr import dataclass
from google.cloud.storage import Bucket, Client
from returns.functions import raise_exception
from returns.pipeline import pipeline
from returns.result import ResultE, safe
from typing_extensions import final

from storage_bucket.constants import DEFAULT_TIMEOUT, TIMEOUT_TYPE


@final
@dataclass(frozen=True, slots=True)
class GetBucket(object):
    """Get a GCP storage bucket."""

    _client = Client

    @pipeline(ResultE)
    def __call__(
        self,
        storage_bucket_name: str,
        timeout: TIMEOUT_TYPE = DEFAULT_TIMEOUT,
    ) -> ResultE[Bucket]:
        """Get the storage bucket."""
        client = self._initialize_client().unwrap()  # type: ignore
        return self._get_bucket(  # type: ignore
            client,
            storage_bucket_name,
            timeout,
        )

    @safe
    def _get_bucket(
        self,
        client: Client,
        bucket_name: str,
        timeout: TIMEOUT_TYPE,
    ) -> Bucket:
        return client.get_bucket(bucket_name)

    @safe
    def _initialize_client(self) -> Client:
        return self._client()


def get_bucket(
    storage_bucket_name: str,
) -> Bucket:
    """Get bucket but return bucket instead of Modal.

    Raise exception when Modal is in failure state.
    """
    return GetBucket()(
        storage_bucket_name=storage_bucket_name,
    ).alt(
        raise_exception,
    ).unwrap()



from attr import dataclass
from google.cloud.storage import Bucket, Client
from returns.functions import raise_exception
from returns.pipeline import pipeline
from returns.result import ResultE, safe
from typing_extensions import final


@final
@dataclass(frozen=True, slots=True)
class CreateBucket(object):
    """Create a gcp storage bucket."""

    _client = Client
    _bucket = Bucket

    @pipeline(ResultE)
    def __call__(
        self,
        storage_bucket_name: str,
        location: str,
        storage_class: str = 'STANDARD',
    ) -> ResultE[Bucket]:
        """List the storage bucket files."""
        client = self._initialize_client().unwrap()  # type: ignore
        bucket = self._initialize_bucket(  # type: ignore
            client,
            storage_bucket_name,
            storage_class,
        ).unwrap()

        return self._create_bucket(client, bucket, location)  # type: ignore

    @safe
    def _create_bucket(
        self,
        client: Client,
        bucket: Bucket,
        location: str,
    ) -> Bucket:
        return client.create_bucket(bucket, location=location)

    @safe
    def _initialize_bucket(
        self, client: Client, name: str, storage_class: str,
    ) -> Bucket:
        bucket = Bucket(client, name=name)
        bucket.storage_class = storage_class
        return bucket

    @safe
    def _initialize_client(self) -> Client:
        return self._client()


def create_bucket(
    storage_bucket_name: str,
    location: str,
    storage_class: str = 'STANDARD',
) -> Bucket:
    """Create bucket but return bucket instead of Modal.

    Raise exception when Modal is in failure state.
    """
    return CreateBucket()(
        storage_bucket_name=storage_bucket_name,
        location=location,
        storage_class=storage_class,
    ).alt(
        raise_exception,
    ).unwrap()



from functools import partial
from typing import Optional

from attr import dataclass
from google.cloud.storage import Bucket
from returns.functions import raise_exception
from returns.pipeline import pipeline
from returns.result import ResultE, safe
from typing_extensions import final

from storage_bucket.constants import TIMEOUT_TYPE
from storage_bucket.get import GetBucket


@final
@dataclass(frozen=True, slots=True)
class DeleteFile(object):
    """
    Deletes a google cloud storage bucket file.

    .. versionadded:: 0.2.0

    :param bucket_name: Name of the google cloud bucket
    :type bucket_name: str

    :param filename: The name of the file to delete
    :type filename: str

    :param generation: ID of the generation(version) of the file to delete
    :type generation: int

    :param timout: The amount of time, in seconds, to wait for the server
    response.

    :return: Result[None, Exception]
    """

    _get_bucket = GetBucket()

    @pipeline(ResultE)
    def __call__(
        self,
        storage_bucket_name: str,
        filename: str,
        generation: Optional[int] = None,
        timeout: Optional[TIMEOUT_TYPE] = None,
    ) -> ResultE[None]:
        """Delete storage bucket file."""
        return self._get_bucket(
            storage_bucket_name,
        ).bind(
            partial(self._delete_file, filename=filename),
        )

    @safe
    def _delete_file(self, bucket: Bucket, filename: str) -> None:
        bucket.delete_blob(filename)


def delete_file(
    storage_bucket_name: str,
    filename: str,
    generation: Optional[int] = None,
) -> None:
    """Delete file with DeleteFile but raise exception on failure."""
    DeleteFile()(
        storage_bucket_name=storage_bucket_name,
        filename=filename,
        generation=generation,
    ).alt(
        raise_exception,
    ).unwrap()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['storage_bucket']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.3.0,<20.0.0',
 'google-cloud-storage>=1.28.1,<2.0.0',
 'returns>=0.13.0,<0.14.0']

setup_kwargs = {
    'name': 'storage-bucket',
    'version': '0.5.1',
    'description': 'Easy to work with Google Cloud Platform Storage Bucket wrapper',
    'long_description': "![test](https://github.com/thomasborgen/storage-bucket/workflows/test/badge.svg)\n[![codecov](https://codecov.io/gh/thomasborgen/storage-bucket/branch/master/graph/badge.svg)](https://codecov.io/gh/thomasborgen/storage-bucket)\n[![Python Version](https://img.shields.io/pypi/pyversions/storage-bucket.svg)](https://pypi.org/project/storage-bucket/)\n[![wemake-python-styleguide](https://img.shields.io/badge/style-wemake-000000.svg)](https://github.com/wemake-services/wemake-python-styleguide)\n\n# Google Cloud Platform Storage Bucket\n\nThis package just aims to make life a little bit easier for people who have to work with google cloud storage bucket.\n\n* [Quickstart](#quickstart)\n* [File/Blob operations](#file/blob-operations)\n  * [Download](#download)\n  * [Upload](#upload)\n  * [List](#list)\n  * [Delete](#delete)\n* [Bucket operations](#bucket-operations)\n  * [Create](#create-bucket)\n  * [Delete](#delete-bucket)\n  * [List Buckets](#list-buckets)\n\n\n## Quickstart:\n\n1. get the package\n  * `pip install storage-bucket`\n2. Download your keyfile and save it as key.json and point to it with env var:\n  * `gcloud iam service-accounts keys create key.json --iam-account your_service_account@your_project.iam.gserviceaccount.com`\n  * `export GOOGLE_APPLICATION_CREDENTIALS='key.json'`\n3. Run some code:\n\n\n```python\nfrom storage_bucket.download_file import DownloadFile, download_file\n\n\ndef use_data_for_something(data):\n    print(data)\n\n\n# Normal way, this might throw exception... handle them yourself.\nmy_data = download_file(\n    'my_bucket',\n    'my_file.txt',\n)\nuse_data_for_something(my_data)\n\n\n# Returns Modal way\n# this will _only_ call use_data_for_something when data is successfully downloaded.\n# so its completely safe.\nDownloadFile()(\n    'my_bucket',\n    'my_file.txt',\n).map(\n    use_data_for_something,  # send data to this function,\n)\n\n```\n\n## File/blob operations\n\n### Download\n\n```python\nfrom storage_bucket.download_file import DownloadFile, download_file\n\nDownloadFile()('bucket', 'filename')\ndownload_file('bucket', 'filename')\n```\n\n### Upload\n```python\nfrom storage_bucket.upload_file import UploadFile, upload_file\n\nUploadFile()(b'data', 'bucket_name', 'filename')\nupload_file(b'data', 'bucket_name', 'filename')\n```\n\n### List\n```python\nfrom storage_bucket.list_files import ListFiles, list_files\n\nListFiles()('bucket')\nlist_files('bucket')\n\nListFiles()('bucket', 'foldername/')\nlist_files('bucket', 'foldername/')\n```\n\n### Delete\n```python\nfrom storage_bucket.delete_file import DeleteFile, delete_file\n\nDeleteFile()('bucketname', 'filename')\ndelete_file('bucketname', 'filename')\n```\n\n## Bucket operations\n\n### Create Bucket\n```python\nfrom storage_bucket.create import CreateBucket, create_bucket\n\nCreateBucket()('bucket-name', 'EU', 'STANDARD')\ncreate_bucket('bucket-name', 'EU', 'STANDARD')\n\n```\n\n### Delete Bucket\n```python\nfrom storage_bucket.delete import DeleteBucket, delete_bucket\n\nDeleteBucket()('bucket-name')\ndelete_bucket('bucket-name')\n\n```\n\n### List Buckets\n```python\nfrom storage_bucket.list import ListBuckets, list_buckets, list_bucket_names\n\nbuckets = ListBuckets()()\nbucket_names = list_bucket_names(buckets.unwrap())\n\nbuckets2 = list_buckets()\nbucket_names2 = list_bucket_names(buckets2)\n```\n\n\n### The use of [Returns](https://github.com/dry-python/returns) library.\n  * Lets us get rid of all exceptions.\n  * Lets us chain stuff so everything looks good.\n  * Lets you use `DownloadFile()(args...).map(dostuff).alt(dostuffonfailure)`\n  * Don't like it? use the matching normal function provided for your convenience.\n\n## Contribution\n\nLike the library and want to help us, check: [CONTRIBUTING.md](https://github.com/thomasborgen/storage-bucket/CONTRIBUTING.md)\n",
    'author': 'Thomas Borgen',
    'author_email': 'thomas@borgenit.no',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/thomasborgen/storage-bucket',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from attr import dataclass
from google.cloud.storage import Bucket, Client
from returns.functions import raise_exception
from returns.pipeline import pipeline
from returns.result import ResultE, safe
from typing_extensions import final

from storage_bucket.constants import DEFAULT_TIMEOUT, TIMEOUT_TYPE


@final
@dataclass(frozen=True, slots=True)
class DeleteBucket(object):
    """Create a gcp storage bucket."""

    _client = Client

    @pipeline(ResultE)
    def __call__(
        self,
        storage_bucket_name: str,
        force: bool = False,
        timeout: TIMEOUT_TYPE = DEFAULT_TIMEOUT,
    ) -> ResultE[None]:
        """List the storage bucket files."""
        client = self._initialize_client().unwrap()  # type: ignore

        bucket = self._get_bucket(  # type: ignore
            client, storage_bucket_name,
        ).unwrap()

        return self._delete(bucket, force, timeout)

    @safe
    def _get_bucket(self, client: Client, name: str) -> Bucket:
        return client.get_bucket(name)

    @safe
    def _delete(
        self,
        bucket: Bucket,
        force: bool,
        timeout: TIMEOUT_TYPE,
    ) -> None:
        # This raises various exceptions.
        bucket.delete(force=force, timeout=timeout)

    @safe
    def _initialize_client(self) -> Client:
        return self._client()


def delete_bucket(
    storage_bucket_name: str,
    force: bool = False,
    timeout: TIMEOUT_TYPE = DEFAULT_TIMEOUT,
) -> None:
    """Delete bucket. Returns None on Success.

    Raise exception when Modal is in failure state.
    """
    return DeleteBucket()(
        storage_bucket_name=storage_bucket_name,
        force=force,
        timeout=timeout,
    ).alt(
        raise_exception,
    ).unwrap()
